@@ -18,8 +18,10 @@
 
 package org.apache.commons.exec;
 
+import java.io.File;
 import java.util.Arrays;
 import java.util.HashMap;
+import java.util.Map;
 
 import junit.framework.AssertionFailedError;
 import junit.framework.TestCase;
@@ -27,15 +29,15 @@ import org.apache.commons.exec.util.StringUtils;
 
 public class CommandLineTest extends TestCase {
 
-    private void assertEquals(String[] expected, String[] actual) {
+    private void assertEquals(final String[] expected, final String[] actual) {
         if (!Arrays.equals(expected, actual)) {
             throw new AssertionFailedError("Arrays not equal");
         }
     }
 
     public void testExecutable() {
-        CommandLine cmdl = new CommandLine("test");
-        assertEquals("test", cmdl.toString());
+        final CommandLine cmdl = new CommandLine("test");
+        assertEquals("[test]", cmdl.toString());
         assertEquals(new String[] {"test"}, cmdl.toStrings());
         assertEquals("test", cmdl.getExecutable());
         assertTrue(cmdl.getArguments().length == 0);
@@ -45,7 +47,7 @@ public class CommandLineTest extends TestCase {
         try {
             new CommandLine("");
             fail("Must throw IllegalArgumentException");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // Expected
         }
     }
@@ -54,7 +56,7 @@ public class CommandLineTest extends TestCase {
         try {
             new CommandLine("   ");
             fail("Must throw IllegalArgumentException");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // Expected
         }
     }
@@ -63,107 +65,107 @@ public class CommandLineTest extends TestCase {
         try {
             new CommandLine((String)null);
             fail("Must throw IllegalArgumentException");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // Expected
         }
     }
 
     public void testAddArgument() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
 
         cmdl.addArgument("foo");
         cmdl.addArgument("bar");
-        assertEquals("test foo bar", cmdl.toString());
+        assertEquals("[test, foo, bar]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "bar"}, cmdl.toStrings());
     }
 
     public void testAddNullArgument() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
 
         cmdl.addArgument(null);
-        assertEquals("test", cmdl.toString());
+        assertEquals("[test]", cmdl.toString());
         assertEquals(new String[] {"test"}, cmdl.toStrings());
     }
 
     public void testAddArgumentWithSpace() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArgument("foo");
         cmdl.addArgument("ba r");
-        assertEquals("test foo \"ba r\"", cmdl.toString());
+        assertEquals("[test, foo, \"ba r\"]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "\"ba r\""}, cmdl.toStrings());
     }
 
     public void testAddArgumentWithQuote() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArgument("foo");
         cmdl.addArgument("ba\"r");
-        assertEquals("test foo 'ba\"r'", cmdl.toString());
+        assertEquals("[test, foo, 'ba\"r']", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "'ba\"r'"}, cmdl.toStrings());
     }
 
     public void testAddArgumentWithQuotesAround() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArgument("\'foo\'");
         cmdl.addArgument("\"bar\"");
         cmdl.addArgument("\"fe z\"");
-        assertEquals("test foo bar \"fe z\"", cmdl.toString());
+        assertEquals("[test, foo, bar, \"fe z\"]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "bar", "\"fe z\""}, cmdl.toStrings());
     }
 
     public void testAddArgumentWithSingleQuote() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
 
         cmdl.addArgument("foo");
         cmdl.addArgument("ba'r");
-        assertEquals("test foo \"ba'r\"", cmdl.toString());
+        assertEquals("[test, foo, \"ba'r\"]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "\"ba\'r\""}, cmdl
                 .toStrings());
     }
 
     public void testAddArgumentWithBothQuotes() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
 
         try {
             cmdl.addArgument("b\"a'r");
             fail("IllegalArgumentException should be thrown");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // OK, expected
         }
     }
 
     public void testAddArguments() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArguments("foo bar");
-        assertEquals("test foo bar", cmdl.toString());
+        assertEquals("[test, foo, bar]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "bar"}, cmdl.toStrings());
     }
 
     public void testAddArgumentsWithQuotes() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArguments("'foo' \"bar\"");
-        assertEquals("test foo bar", cmdl.toString());
+        assertEquals("[test, foo, bar]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "bar"}, cmdl.toStrings());
     }
 
     public void testAddArgumentsWithQuotesAndSpaces() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArguments("'fo o' \"ba r\"");
-        assertEquals("test \"fo o\" \"ba r\"", cmdl.toString());
+        assertEquals("[test, \"fo o\", \"ba r\"]", cmdl.toString());
         assertEquals(new String[] {"test", "\"fo o\"", "\"ba r\""}, cmdl
                 .toStrings());
     }
 
     public void testAddArgumentsArray() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArguments(new String[] {"foo", "bar"});
-        assertEquals("test foo bar", cmdl.toString());
+        assertEquals("[test, foo, bar]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "bar"}, cmdl.toStrings());
     }
 
     public void testAddArgumentsArrayNull() {
-        CommandLine cmdl = new CommandLine("test");
+        final CommandLine cmdl = new CommandLine("test");
         cmdl.addArguments((String[]) null);
-        assertEquals("test", cmdl.toString());
+        assertEquals("[test]", cmdl.toString());
         assertEquals(new String[] {"test"}, cmdl.toStrings());
     }
 
@@ -174,12 +176,12 @@ public class CommandLineTest extends TestCase {
      */
     public void testAddTwoArguments() {
 
-        CommandLine userAddCL1 = new CommandLine("useradd");
+        final CommandLine userAddCL1 = new CommandLine("useradd");
         userAddCL1.addArgument("-g");
         userAddCL1.addArgument("tomcat");
         userAddCL1.addArgument("foo");
 
-        CommandLine userAddCL2 = new CommandLine("useradd");
+        final CommandLine userAddCL2 = new CommandLine("useradd");
         userAddCL2.addArgument("-g").addArgument("tomcat");
         userAddCL2.addArgument("foo");
 
@@ -187,14 +189,14 @@ public class CommandLineTest extends TestCase {
     }
 
     public void testParseCommandLine() {
-        CommandLine cmdl = CommandLine.parse("test foo bar");
-        assertEquals("test foo bar", cmdl.toString());
+        final CommandLine cmdl = CommandLine.parse("test foo bar");
+        assertEquals("[test, foo, bar]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "bar"}, cmdl.toStrings());
     }
 
     public void testParseCommandLineWithQuotes() {
-        CommandLine cmdl = CommandLine.parse("test \"foo\" \'ba r\'");
-        assertEquals("test foo \"ba r\"", cmdl.toString());
+        final CommandLine cmdl = CommandLine.parse("test \"foo\" \'ba r\'");
+        assertEquals("[test, foo, \"ba r\"]", cmdl.toString());
         assertEquals(new String[] {"test", "foo", "\"ba r\""}, cmdl.toStrings());
     }
 
@@ -202,7 +204,7 @@ public class CommandLineTest extends TestCase {
         try {
             CommandLine.parse("test \"foo bar");
             fail("IllegalArgumentException must be thrown due to uneven quotes");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // Expected
         }
     }
@@ -211,7 +213,7 @@ public class CommandLineTest extends TestCase {
         try {
             CommandLine.parse(null);
             fail("IllegalArgumentException must be thrown due to incorrect command line");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // Expected
         }
     }
@@ -220,7 +222,7 @@ public class CommandLineTest extends TestCase {
         try {
             CommandLine.parse("  ");
             fail("IllegalArgumentException must be thrown due to incorrect command line");
-        } catch (IllegalArgumentException e) {
+        } catch (final IllegalArgumentException e) {
             // Expected
         }
     }
@@ -229,14 +231,14 @@ public class CommandLineTest extends TestCase {
     /**
      * A command line parsing puzzle from Tino Schoellhorn - ImageMagix expects
      * a "500x>" parameter (including quotes) and it is simply not possible to
-     * do that withoud adding a space, e.g. "500x> ".
+     * do that without adding a space, e.g. "500x> ".
      */
     public void testParseComplexCommandLine1() {
-        HashMap substitutionMap = new HashMap();
+        final HashMap substitutionMap = new HashMap();
         substitutionMap.put("in", "source.jpg");
         substitutionMap.put("out", "target.jpg");
-        CommandLine cmdl = CommandLine.parse("cmd /C convert ${in} -resize \"\'500x> \'\" ${out}", substitutionMap);
-        assertEquals("cmd /C convert source.jpg -resize \"500x> \" target.jpg", cmdl.toString());        
+        final CommandLine cmdl = CommandLine.parse("cmd /C convert ${in} -resize \"\'500x> \'\" ${out}", substitutionMap);
+        assertEquals("[cmd, /C, convert, source.jpg, -resize, \"500x> \", target.jpg]", cmdl.toString());
     }
 
     /**
@@ -246,14 +248,14 @@ public class CommandLineTest extends TestCase {
      */
     public void testParseComplexCommandLine2() {
 
-        String commandline = "./script/jrake cruise:publish_installers "
+        final String commandline = "./script/jrake cruise:publish_installers "
             + "INSTALLER_VERSION=unstable_2_1 "
             + "INSTALLER_PATH=\"/var/lib/ cruise-agent/installers\" "
             + "INSTALLER_DOWNLOAD_SERVER=\'something\' "
             + "WITHOUT_HELP_DOC=true";
 
-        CommandLine cmdl = CommandLine.parse(commandline);
-        String[] args = cmdl.getArguments();
+        final CommandLine cmdl = CommandLine.parse(commandline);
+        final String[] args = cmdl.getArguments();
         assertEquals(args[0], "cruise:publish_installers");
         assertEquals(args[1], "INSTALLER_VERSION=unstable_2_1");
         // assertEquals(args[2], "INSTALLER_PATH=\"/var/lib/ cruise-agent/installers\"");
@@ -261,17 +263,31 @@ public class CommandLineTest extends TestCase {
         assertEquals(args[4], "WITHOUT_HELP_DOC=true");
     }
 
+    /**
+     * Test the following command line
+     *
+     * cmd.exe /C c:\was51\Web Sphere\AppServer\bin\versionInfo.bat
+     */
+    public void testParseRealLifeCommandLine_1() {
+
+        final String commandline = "cmd.exe /C \"c:\\was51\\Web Sphere\\AppServer\\bin\\versionInfo.bat\"";
+
+        final CommandLine cmdl = CommandLine.parse(commandline);
+        final String[] args = cmdl.getArguments();
+        assertEquals("/C", args[0]);
+        assertEquals("\"c:\\was51\\Web Sphere\\AppServer\\bin\\versionInfo.bat\"", args[1]);
+    }
+        
    /**
     * Create a command line with pre-quoted strings to test SANDBOX-192,
     * e.g. "runMemorySud.cmd", "10", "30", "-XX:+UseParallelGC", "\"-XX:ParallelGCThreads=2\""
     */
     public void testComplexAddArgument() {
-        CommandLine cmdl = new CommandLine("runMemorySud.cmd");
+        final CommandLine cmdl = new CommandLine("runMemorySud.cmd");
         cmdl.addArgument("10", false);
         cmdl.addArgument("30", false);
         cmdl.addArgument("-XX:+UseParallelGC", false);
         cmdl.addArgument("\"-XX:ParallelGCThreads=2\"", false);
-        assertEquals("runMemorySud.cmd 10 30 -XX:+UseParallelGC \"-XX:ParallelGCThreads=2\"", cmdl.toString());
         assertEquals(new String[] {"runMemorySud.cmd", "10", "30", "-XX:+UseParallelGC", "\"-XX:ParallelGCThreads=2\""}, cmdl.toStrings());
     }
 
@@ -280,9 +296,8 @@ public class CommandLineTest extends TestCase {
      * e.g. "runMemorySud.cmd", "10", "30", "-XX:+UseParallelGC", "\"-XX:ParallelGCThreads=2\""
      */
      public void testComplexAddArguments1() {
-         CommandLine cmdl = new CommandLine("runMemorySud.cmd");
+         final CommandLine cmdl = new CommandLine("runMemorySud.cmd");
          cmdl.addArguments(new String[] {"10", "30", "-XX:+UseParallelGC", "\"-XX:ParallelGCThreads=2\""}, false);
-         assertEquals("runMemorySud.cmd 10 30 -XX:+UseParallelGC \"-XX:ParallelGCThreads=2\"", cmdl.toString());
          assertEquals(new String[] {"runMemorySud.cmd", "10", "30", "-XX:+UseParallelGC", "\"-XX:ParallelGCThreads=2\""}, cmdl.toStrings());
      }
 
@@ -293,9 +308,8 @@ public class CommandLineTest extends TestCase {
      * don't know if this is a bug or a feature.
      */
      public void testComplexAddArguments2() {
-         CommandLine cmdl = new CommandLine("runMemorySud.cmd");
+         final CommandLine cmdl = new CommandLine("runMemorySud.cmd");
          cmdl.addArguments("10 30 -XX:+UseParallelGC '\"-XX:ParallelGCThreads=2\"'", false);
-         assertEquals("runMemorySud.cmd 10 30 -XX:+UseParallelGC \"-XX:ParallelGCThreads=2\"", cmdl.toString());
          assertEquals(new String[] {"runMemorySud.cmd", "10", "30", "-XX:+UseParallelGC", "\"-XX:ParallelGCThreads=2\""}, cmdl.toStrings());
      }
 
@@ -304,13 +318,15 @@ public class CommandLineTest extends TestCase {
      */
     public void testCommandLineParsingWithExpansion1() {
 
-        CommandLine cmdl = null;
+        CommandLine cmdl;
 
-        HashMap substitutionMap = new HashMap();
+        final HashMap substitutionMap = new HashMap();
         substitutionMap.put("JAVA_HOME", "/usr/local/java");
         substitutionMap.put("appMainClass", "foo.bar.Main");
+        substitutionMap.put("file1", new File("./pom.xml"));
+        substitutionMap.put("file2", new File(".\\temp\\READ ME.txt"));
 
-        HashMap incompleteMap = new HashMap();
+        final HashMap incompleteMap = new HashMap();
         incompleteMap.put("JAVA_HOME", "/usr/local/java");
 
         // do not pass substitution map
@@ -334,6 +350,10 @@ public class CommandLineTest extends TestCase {
         assertTrue(cmdl.getExecutable().indexOf("${JAVA_HOME}") < 0 );
         assertTrue(cmdl.getExecutable().indexOf("local") > 0 );
         assertEquals(new String[] {"${appMainClass}"}, cmdl.getArguments());
+
+        // pass a file
+        cmdl = CommandLine.parse("${JAVA_HOME}/bin/java ${appMainClass} ${file1} ${file2}", substitutionMap);
+        assertTrue(cmdl.getExecutable().indexOf("${file}") < 0 );
     }
 
     /**
@@ -347,7 +367,7 @@ public class CommandLineTest extends TestCase {
         String[] result;
 
         // build the user supplied parameters
-        HashMap substitutionMap = new HashMap();
+        final HashMap substitutionMap = new HashMap();
         substitutionMap.put("JAVA_HOME", "C:\\Programme\\jdk1.5.0_12");
         substitutionMap.put("appMainClass", "foo.bar.Main");
 
@@ -368,16 +388,16 @@ public class CommandLineTest extends TestCase {
         assertEquals(StringUtils.fixFileSeparatorChar("C:\\Programme\\jdk1.5.0_12\\bin\\java"), result[0]);
         assertEquals("-class", result[1]);
         assertEquals("foo.bar.Main", result[2]);
-        assertEquals("C:\\Document And Settings\\documents\\432431.pdf", result[3]);
+        assertEquals("\"C:\\Document And Settings\\documents\\432431.pdf\"", result[3]);
 
         // verify the first command line again but by
         // accessing the executable and arguments directly
-        String executable = cmdl.getExecutable();
-        String[] arguments = cmdl.getArguments();
+        final String executable = cmdl.getExecutable();
+        final String[] arguments = cmdl.getArguments();
         assertEquals(StringUtils.fixFileSeparatorChar("C:\\Programme\\jdk1.5.0_12\\bin\\java"), executable);
         assertEquals("-class", arguments[0]);
         assertEquals("foo.bar.Main", arguments[1]);
-        assertEquals("C:\\Document And Settings\\documents\\432431.pdf", arguments[2]);
+        assertEquals("\"C:\\Document And Settings\\documents\\432431.pdf\"", arguments[2]);
 
         // build the second command line with updated parameters resulting in  a different command line
         substitutionMap.put("file", "C:\\Document And Settings\\documents\\432432.pdf");        
@@ -385,18 +405,18 @@ public class CommandLineTest extends TestCase {
         assertEquals(StringUtils.fixFileSeparatorChar("C:\\Programme\\jdk1.5.0_12\\bin\\java"), result[0]);
         assertEquals("-class", result[1]);
         assertEquals("foo.bar.Main", result[2]);
-        assertEquals("C:\\Document And Settings\\documents\\432432.pdf", result[3]);                
+        assertEquals("\"C:\\Document And Settings\\documents\\432432.pdf\"", result[3]);                
     }
 
-    public void testCommandLineParsingWithExpansion3(){
-        CommandLine cmdl = CommandLine.parse("AcroRd32.exe");
+    public void testCommandLineParsingWithExpansion3() {
+        final CommandLine cmdl = CommandLine.parse("AcroRd32.exe");
         cmdl.addArgument("/p");
         cmdl.addArgument("/h");
-        cmdl.addArgument("${file}");
-        HashMap params = new HashMap();
+        cmdl.addArgument("${file}", false);
+        final HashMap params = new HashMap();
         params.put("file", "C:\\Document And Settings\\documents\\432432.pdf");
         cmdl.setSubstitutionMap(params);
-        String[] result = cmdl.toStrings();
+        final String[] result = cmdl.toStrings();
         assertEquals("AcroRd32.exe", result[0]);
         assertEquals("/p", result[1]);
         assertEquals("/h", result[2]);
@@ -404,24 +424,123 @@ public class CommandLineTest extends TestCase {
         
     }
     /**
-     * Test the toString() method
+     * Test the toString() method.
+     *
+     * @throws Exception the test failed
      */
     public void testToString() throws Exception {
         CommandLine cmdl;
-        HashMap params = new HashMap();
+        final HashMap params = new HashMap();
 
         // use no arguments
         cmdl = CommandLine.parse("AcroRd32.exe", params);
-        assertEquals(cmdl.toString(), "AcroRd32.exe");
+        assertEquals("[AcroRd32.exe]", cmdl.toString());
 
         // use an argument containing spaces
         params.put("file", "C:\\Document And Settings\\documents\\432432.pdf");
         cmdl = CommandLine.parse("AcroRd32.exe /p /h '${file}'", params);
-        assertEquals(cmdl.toString(), "AcroRd32.exe /p /h \"C:\\Document And Settings\\documents\\432432.pdf\"");
+        assertEquals("[AcroRd32.exe, /p, /h, \"C:\\Document And Settings\\documents\\432432.pdf\"]", cmdl.toString());
 
         // use an argument without spaces
         params.put("file", "C:\\documents\\432432.pdf");
         cmdl = CommandLine.parse("AcroRd32.exe /p /h '${file}'", params);
-        assertEquals(cmdl.toString(), "AcroRd32.exe /p /h C:\\documents\\432432.pdf");
+        assertEquals("[AcroRd32.exe, /p, /h, C:\\documents\\432432.pdf]", cmdl.toString());
     }
+
+    /**
+     * Test that toString() produces output that is useful for troubleshooting.
+     *
+     * @throws Exception the test failed
+     */
+    public void testToStringTroubleshooting() throws Exception {
+        System.out.println("testToStringTroubleshooting");
+        // On HP-UX quotes handling leads to errors,
+        // also usage of quotes isn't mandatory on other platforms too
+        // so it probably should work correctly either way.
+        final CommandLine cmd1 = new CommandLine("sh").addArgument("-c")
+                .addArgument("echo 1", false);
+        final CommandLine cmd2 = new CommandLine("sh").addArgument("-c")
+                .addArgument("echo").addArgument("1");
+        System.out.println("cmd1: " + cmd1.toString());
+        System.out.println("cmd2: " + cmd2.toString());
+        assertTrue("toString() is useful for troubleshooting",
+                !cmd1.toString().equals(cmd2.toString()));
+    }
+
+    /**
+     * Some complex real-life command line from
+     * http://blogs.msdn.com/b/astebner/archive/2005/12/13/503471.aspx
+     */
+    public void _testExec36_1() throws Exception {
+
+        CommandLine cmdl;
+
+        final String line = "./script/jrake "
+          + "cruise:publish_installers "
+          + "INSTALLER_VERSION=unstable_2_1 "
+          + "INSTALLER_PATH=\"/var/lib/cruise-agent/installers\" "
+          + "INSTALLER_DOWNLOAD_SERVER='something'"
+          + "WITHOUT_HELP_DOC=true";
+
+        cmdl = CommandLine.parse(line);
+        final String[] args = cmdl.toStrings();
+        assertEquals("./script/jrake", args[0]);
+        assertEquals("cruise:publish_installers", args[1]);
+        assertEquals("INSTALLER_VERSION=unstable_2_1", args[2]);
+        assertEquals("INSTALLER_PATH=\"/var/lib/cruise-agent/installers\"", args[3]);
+        assertEquals("INSTALLER_DOWNLOAD_SERVER='something'", args[4]);
+        assertEquals("WITHOUT_HELP_DOC=true", args[5]);
+    }
+
+    /**
+     * Some complex real-life command line from
+     * http://blogs.msdn.com/b/astebner/archive/2005/12/13/503471.aspx
+     */
+    public void _testExec36_2() {
+
+        CommandLine cmdl;
+
+        final String line = "dotnetfx.exe"
+                + " /q:a "
+                + "/c:\"install.exe /l \"\"c:\\Documents and Settings\\myusername\\Local Settings\\Temp\\netfx.log\"\" /q\"";
+
+        cmdl = CommandLine.parse(line);
+        final String[] args = cmdl.toStrings();
+        assertEquals("dotnetfx.exe", args[0]);
+        assertEquals("/q:a", args[1]);
+        assertEquals("/c:\"install.exe /l \"\"c:\\Documents and Settings\\myusername\\Local Settings\\Temp\\netfx.log\"\" /q\"", args[2] );
+    }
+
+    /**
+     * Test the following command line
+     *
+     * C:\CVS_DB\WeightsEngine /f WeightsEngine.mak CFG="WeightsEngine - Win32Release"
+     */
+    public void _testExec36_3() {
+
+        final String commandline = "C:\\CVS_DB\\WeightsEngine /f WeightsEngine.mak CFG=\"WeightsEngine - Win32Release\"";
+
+        final CommandLine cmdl = CommandLine.parse(commandline);
+        final String[] args = cmdl.getArguments();
+        assertEquals("/f", args[0]);
+        assertEquals("WeightsEngine.mak", args[1]);
+        assertEquals("CFG=\"WeightsEngine - Win32Release\"", args[2]);
+    }
+
+    public void testCopyConstructor()
+    {
+        final Map map = new HashMap();
+        map.put("bar", "bar");
+        final CommandLine other = new CommandLine("test");
+        other.addArgument("foo");
+        other.setSubstitutionMap(map);
+
+        final CommandLine cmdl = new CommandLine(other);
+        assertEquals(other.getExecutable(), cmdl.getExecutable());
+        assertEquals(other.getArguments(), cmdl.getArguments());
+        assertEquals(other.isFile(), cmdl.isFile());
+        assertEquals(other.getSubstitutionMap(), cmdl.getSubstitutionMap());
+
+    }
+
 }
